/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2009 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.connectors.ws;

import java.util.Map;
import java.util.logging.Logger;

import javax.xml.ws.BindingProvider;

import org.apache.cxf.message.Message;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.bf.BindingFactoryClassLoader;

/**
 * A super class for WS stubs. Concrete classes are generated on the fly.
 */
public abstract class AbstractWsStubContent implements WsStubAttributes,
		BindingController, LifeCycleController {

	Logger log = Logger.getLogger(AbstractWsStubContent.class
			.getCanonicalName());

	/**
	 * Name of the 'remoteRef' client interface.
	 */
	public static String REMOTE_REF_FIELD_NAME = "remoteRef";

	/**
	 * A ref to a stub object. This field is initialized when this component is
	 * started, and its concrete implementation might depend on the presence of
	 * a WSDL file, on an optional URI, and so on.
	 */
	protected Object remoteRef;
	/**
	 * The uri used to bind the remoteRef.
	 */
	private String uri;
	/**
	 * The wsdlElement to configure the binding.
	 */
	private String wsdlElement;
	/**
	 * The wsdlLocation to configure the binding.
	 */
	private String wsdlLocation;
	private String fcState;
	private Class<?> serviceClass;
	private BindingFactoryClassLoader cl;
	/**
	 * The CXF client recovered by the objects instantiated by pre-generated
	 * stubs via wsdl2java. This client is initialized in
	 * {@link AbstractWsStubContent#resolveReference()} .
	 */
	private org.apache.cxf.endpoint.Client client;

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#getUri()
	 */
	public String getUri() {
		return this.uri;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#getWsdlElement()
	 */
	public String getWsdlElement() {
		return this.wsdlElement;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#setUri(java.lang.String)
	 */
	public void setUri(String uri) {
		this.uri = uri;

	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#setWsdlElement(java.lang.String)
	 */
	public void setWsdlElement(String wsdlElement) {
		this.wsdlElement = wsdlElement;
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
	 *      java.lang.Object)
	 */
	public void bindFc(String clientItfName, Object serverItf)
			throws NoSuchInterfaceException, IllegalBindingException,
			IllegalLifeCycleException {
		if (clientItfName.equalsIgnoreCase(REMOTE_REF_FIELD_NAME)) {
			this.remoteRef = serverItf;
			return;
		}
		throw new NoSuchInterfaceException("Cannot find client interface "
				+ clientItfName);
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#listFc()
	 */
	public String[] listFc() {
		return new String[] { REMOTE_REF_FIELD_NAME };
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	 */
	public Object lookupFc(String clientItfName)
			throws NoSuchInterfaceException {
		Object res = null;
		if (clientItfName.equalsIgnoreCase(REMOTE_REF_FIELD_NAME)) {
			res = this.remoteRef;
		}
		return res;
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	 */
	public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {
		if (clientItfName.equalsIgnoreCase(REMOTE_REF_FIELD_NAME)) {
			this.remoteRef = null;
			return;
		}
		throw new NoSuchInterfaceException("Cannot find client interface "
				+ clientItfName);
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return this.fcState;

	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {

		this.remoteRef = resolveReference();

		this.fcState = LifeCycleController.STARTED;
		log.fine("WebService stub started");
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {

		this.remoteRef = null;
		this.fcState = LifeCycleController.STOPPED;
		log.fine("WebService stub stopped");

	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#getServiceClass()
	 */
	public Class<?> getServiceClass() {
		return this.serviceClass;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#setServiceClass(java.lang.Class)
	 */
	public void setServiceClass(Class<?> clazz) {
		this.serviceClass = clazz;

	}

	/**
	 * Build a CXF client using the {@link DefaultResolutionStrategy}.
	 * 
	 * @return the object instantiated by the
	 *         {@link DefaultResolutionStrategy#resolve()} method.
	 */
	protected Object resolveReference() {

		WsStubResolutionStrategy strategy = new DefaultResolutionStrategy(uri,
				wsdlElement, wsdlLocation, serviceClass, cl);
		Object ref = strategy.resolve();

		client = org.apache.cxf.frontend.ClientProxy.getClient(ref);

		/*
		 * to make the client request parameters thread-local
		 */
		client.setThreadLocalRequestContext(true);

		/*
		 * If the uri has been changed because of a reconfiguraiton, orbecuase
		 * the uri given in the wsdl must be modified,cahnge the corresponding
		 * attributes in the request.
		 */
		if (uri != null) {
			final Map<String, Object> requestContext = client
					.getRequestContext();
			// XXX is this taken into account?
			requestContext.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
					this.uri);
			/*
			 * set the URI to which messages are sent: this is CXF specific, but
			 * internally this key is mapped to JaxWs properties.
			 */
			requestContext.put(Message.ENDPOINT_ADDRESS, uri);

			log.info("URI: " + uri);
		}
		// client.getInInterceptors().add(new LoggingInInterceptor());
		// client.getOutInterceptors().add(new LoggingOutInterceptor());

		return ref;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#getClassLoader()
	 */
	public BindingFactoryClassLoader getClassLoader() {
		return this.cl;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#setClassLoader(org.objectweb.fractal.bf.BindingFactoryClassLoader)
	 */
	public void setClassLoader(BindingFactoryClassLoader cl) {
		this.cl = cl;

	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#getWsdlLocation()
	 */
	public String getWsdlLocation() {
		return wsdlLocation;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsStubAttributes#setWsdlLocation(java.lang.String)
	 */
	public void setWsdlLocation(String uri) {
		this.wsdlLocation = uri;
	}
}
