/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2009 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.connectors.ws;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.bf.BindingFactoryClassLoader;
import org.objectweb.fractal.bf.BindingFactoryException;
import org.objectweb.fractal.bf.BindingFactoryPlugin;
import org.objectweb.fractal.bf.SkeletonGenerationException;
import org.objectweb.fractal.bf.StubGenerationException;
import org.objectweb.fractal.bf.StubGenerationUtil;
import org.objectweb.fractal.bf.proxies.ASMStubGenerator;
import org.objectweb.fractal.util.Fractal;

/**
 * A WS-Connector can export and bind Fractal interfaces via web services.
 * 
 */
public class WsConnector implements
		BindingFactoryPlugin<WsExportHints, WsBindHints> {

	Logger log = Logger.getLogger(WsConnector.class.getCanonicalName());
	/**
	 * The {@link TypeFactory} used by this plugin.
	 */
	private TypeFactory typeFactory;
	/**
	 * The {@link GenericFactory} used by this plugin.
	 */
	private GenericFactory cf;

	private BindingFactoryClassLoader cl;

	/**
	 * Default 0-args constructor.
	 */
	public WsConnector() {
		try {
			Component bootstrap = Fractal.getBootstrapComponent();
			typeFactory = Fractal.getTypeFactory(bootstrap);
			cf = Fractal.getGenericFactory(bootstrap);
		} catch (Exception e) {
			throw new WsConnectorException(
					"Could not initialize Fractal bootstrap components.", e);
		}
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#createSkel(java.lang.String,
	 *      java.lang.Object, org.objectweb.fractal.api.Component,
	 *      org.objectweb.fractal.bf.ExportHints)
	 */
	public Component createSkel(String interfaceName, Object serverItf,
			Component owner, WsExportHints wsExportHints)
			throws SkeletonGenerationException {
		log.info("WsConnector - Creating skeleton..");

		String uri = wsExportHints.getUri();

		if (uri == null) {
			log.warning("Export hints didn't specify an uri, going to use "
					+ WsConnectorConstants.DEFAULT_URI);
			uri = WsConnectorConstants.DEFAULT_URI;
		}

		Component wsSkeleton = null;
		try {

			ComponentType wsSkeletonType = typeFactory
					.createFcType(new InterfaceType[] {
							typeFactory.createFcItfType("attribute-controller",
									WsSkeletonAttributes.class
											.getCanonicalName(), false, false,
									false),
							typeFactory.createFcItfType("delegate",
									"org.objectweb.fractal.api.Component",
									true, true, false) });
			wsSkeleton = cf.newFcInstance(wsSkeletonType, "primitive",
					WsSkeletonContent.class.getCanonicalName());

			InterfaceType it = (InterfaceType) ((Interface) serverItf)
					.getFcItfType();
			if (getBindingFactoryClassLoader() == null) {
				throw new IllegalStateException(
						"The BindingFactoryClassLoader was not set");
			}
			Class<?> serviceClass = this.cl.loadClass(it.getFcItfSignature());

			WsSkeletonAttributes wsServiceAttributes = (WsSkeletonAttributes) Fractal
					.getAttributeController(wsSkeleton);
			wsServiceAttributes.setUri(uri);
			wsServiceAttributes.setServiceClass(serviceClass);
			wsServiceAttributes.setInterfaceName(interfaceName);

		} catch (Exception e) {
			throw new SkeletonGenerationException(
					"Some error occurred while exporting using the Ws/CXF plugin ",
					e);
		}

		return wsSkeleton;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getExportHints(Map)
	 */
	public WsExportHints getExportHints(Map<String, Object> initialHints) {
		WsExportHints wsExportHints = new WsExportHints();
		wsExportHints.setUri((String) initialHints
				.get(WsConnectorConstants.URI));
		log.fine("WsExportHints: " + wsExportHints);
		log.warning("WsExportHints do not support "
				+ WsConnectorConstants.WSDL_ELEMENT + " specification yet.");
		return wsExportHints;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getPluginIdentifier()
	 */
	public String getPluginIdentifier() {
		return "ws";
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getBindHints(Map)
	 */
	public WsBindHints getBindHints(Map<String, Object> initialHints) {
		System.err.println("Raw hints: " + initialHints);
		WsBindHints wsBindHints = new WsBindHints();
		wsBindHints.setUri((String) initialHints.get(WsConnectorConstants.URI));
		wsBindHints.setWsdlElement((String) initialHints
				.get(WsConnectorConstants.WSDL_ELEMENT));
		if (initialHints.containsKey(WsConnectorConstants.WSDL_LOCATION)) {
			List<String> wsdLocationXml = (List<String>) initialHints
					.get(WsConnectorConstants.WSDL_LOCATION);
			if (wsdLocationXml != null && wsdLocationXml.size() > 0)
				wsBindHints.setWsdlLocation(wsdLocationXml.get(0));
		}
		log.fine("WsBindHints: " + wsBindHints);
		return wsBindHints;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#createStub(org.objectweb.fractal.api.Component,
	 *      java.lang.String, org.objectweb.fractal.bf.BindHints)
	 */
	public Component createStub(Component client, String itfname,
			WsBindHints wsBindHints) throws StubGenerationException {

		if (wsBindHints == null) {
			throw new StubGenerationException(
					"WsBindHints to create stub are null");
		}

		Component stubComponent = null;
		InterfaceType businessInterfaceItfType = null;

		try {
			ComponentType itf = (ComponentType) (client).getFcType();
			businessInterfaceItfType = itf.getFcInterfaceType(itfname);
			Class<?> wsStubItf = null;

			if (getBindingFactoryClassLoader() == null) {
				throw new IllegalStateException(
						"The BindingFactoryClassLoader was not set");
			}
			wsStubItf = this.cl.loadClass(businessInterfaceItfType
					.getFcItfSignature());
			// create the impl
			final Object stubImpl = ASMStubGenerator.generateNewInstance(
					wsStubItf, AbstractWsStubContent.class,
					getBindingFactoryClassLoader());

			InterfaceType stubReversedInterfaceType = StubGenerationUtil
					.symmetricInterfaceType(businessInterfaceItfType);
			log.finer("Interface type for stub component created: "
					+ stubReversedInterfaceType);

			/*
			 * the AC type for ws stub
			 */
			InterfaceType acIType = typeFactory.createFcItfType(
					"attribute-controller", WsStubAttributes.class
							.getCanonicalName(), false, false, false);

			ComponentType stubComponentType = typeFactory
					.createFcType(new InterfaceType[] { acIType,
							stubReversedInterfaceType });

			log.finer("Component type for stub component created: "
					+ stubComponentType);

			// cl, membrane descr
			Object[] ctrlDesc = new Object[] { this.cl, "primitive" };
			log.finest("Controller descriptor for stub component: "
					+ Arrays.deepToString(ctrlDesc));
			stubComponent = cf.newFcInstance(stubComponentType, ctrlDesc,
					stubImpl);

			WsStubAttributes wssa = (WsStubAttributes) Fractal
					.getAttributeController(stubComponent);
			wssa.setUri(wsBindHints.getUri());
			wssa.setWsdlElement(wsBindHints.getWsdlElement());
			wssa.setWsdlLocation(wsBindHints.getWsdlLocation());
			wssa.setServiceClass(wsStubItf);
			wssa.setClassLoader(this.cl);

		} catch (Exception e) {
			e.printStackTrace();
			throw new StubGenerationException(
					"Some error occurred while creating the WS stub component",
					e);
		}

		return stubComponent;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#finalizeSkeleton(org.objectweb.fractal.api.Component,
	 *      java.util.Map)
	 */
	public void finalizeSkeleton(Component skel,
			Map<String, String> finalizationHints)
			throws BindingFactoryException {

	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#finalizeStub(org.objectweb.fractal.api.Component,
	 *      java.util.Map)
	 */
	public void finalizeStub(Component stub, Map<String, Object> hints)
			throws BindingFactoryException {
	}

	/**
	 * 
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getBindingFactoryClassLoader()
	 */
	public BindingFactoryClassLoader getBindingFactoryClassLoader() {
		return this.cl;
	}

	/**
	 * 
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#setBindingFactoryClassLoader(java.lang.ClassLoader)
	 */
	public void setBindingFactoryClassLoader(BindingFactoryClassLoader cl) {
		this.cl = cl;
	}

}
