/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2009 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.connectors.ws;

import java.util.logging.Logger;

import org.apache.cxf.aegis.AegisContext;
import org.apache.cxf.aegis.databinding.AegisDatabinding;
import org.apache.cxf.endpoint.Server;
import org.apache.cxf.frontend.ServerFactoryBean;
import org.apache.cxf.service.invoker.BeanInvoker;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.bf.AbstractSkeleton;

/**
 * The content implementation of Web Service skeleton components.
 */
public class WsSkeletonContent extends AbstractSkeleton implements
		WsSkeletonAttributes, LifeCycleController {

	Logger log = Logger.getLogger(WsSkeletonContent.class.getCanonicalName());

	/**
	 * the fcState of this component.
	 */
	private String fcState;
	/**
	 * The server factory bean.
	 */
	private final ServerFactoryBean wsServerFactoryBean;
	/**
	 * The service class used by the ServerFactoryBean
	 */
	private Class<?> serviceClass;

	/**
	 * A reference to the server bean. This reference is initialized during the
	 * {@link WsSkeletonContent#startFc()} method.
	 */
	private Server server;

	private String itfName;
	/**
	 * The URI where the Web Service will be made available.
	 */
	private String uri;

	public WsSkeletonContent() {
		wsServerFactoryBean = new ServerFactoryBean();
		/*
		 * the databinding used by this plugin. Should be this configurable,
		 * something must be done in the attribute controller and in the this
		 * code.
		 */
		AegisDatabinding aDB = new AegisDatabinding();
		final AegisContext aegisContext = new AegisContext();
		aegisContext.setWriteXsiTypes(true);
		aDB.setAegisContext(aegisContext);

		wsServerFactoryBean.getServiceFactory().setDataBinding(aDB);
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return this.fcState;
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {
		wsServerFactoryBean.setAddress(this.uri);
		log.fine("ServerFactoryBean will expose web service at URI: "
				+ this.uri);
		wsServerFactoryBean.setServiceClass(this.serviceClass);
		try {

			final Object delegateItf = delegate.getFcInterface(itfName);

			BeanInvoker invoker = new BeanInvoker(delegateItf);
			wsServerFactoryBean.setInvoker(invoker);
			this.server = wsServerFactoryBean.create();
			log.info("WSDL available at the address: " + uri + "?wsdl");
			this.fcState = LifeCycleController.STARTED;

		} catch (NoSuchInterfaceException e) {
			IllegalLifeCycleException ilce = new IllegalLifeCycleException(
					"Cannot start the ws skeleton");
			ilce.initCause(e);
			throw ilce;
		}

	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {
		this.server.stop();

		log.fine("WebService skeleton stopped");

		this.fcState = LifeCycleController.STOPPED;

	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsSkeletonAttributes#setUri(java.lang.String)
	 */
	public void setUri(String uri) {
		this.uri = uri;
		log.info(" WS skeleton configured to use URI: " + uri);
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsSkeletonAttributes#setServiceClass(java.lang.Class)
	 */
	public void setServiceClass(Class<?> serviceClass) {
		this.serviceClass = serviceClass;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsSkeletonAttributes#setInterfaceName(java.lang.String)
	 */
	public void setInterfaceName(String itfName) {
		this.itfName = itfName;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsSkeletonAttributes#getUri()
	 */
	public String getUri() {
		return this.uri;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsSkeletonAttributes#getInterfaceName()
	 */
	public String getInterfaceName() {
		return this.itfName;
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsSkeletonAttributes#getServiceClass()
	 */
	public Class<?> getServiceClass() {
		return this.serviceClass;
	}
}
